@@ -63,7 +63,7 @@
 #include <libkmod.h>
 #include "xf86drm.h"
 #include "xf86drmMode.h"
-#include "json.h"
+#include "json-parser/json.h"
 
 static inline void freep(void *);
 static inline void fclosep(FILE **);
@@ -157,8 +157,6 @@ struct pme_dev {
 
 
 static bool is_file(char *file);
-static bool is_dir(char *directory);
-static bool is_dir_empty(char *directory);
 static bool is_link(char *file);
 static bool is_module_loaded(const char *module);
 static bool get_nvidia_driver_version(int *major, int *minor, int *extra);
@@ -354,23 +352,6 @@ static bool pci_device_is_boot_vga(struct pci_dev *info) {
 }
 
 
-/* Trim string in place */
-static void trim(char *str) {
-    char *pointer = str;
-    int len = strlen(pointer);
-
-    while(isspace(pointer[len - 1]))
-        pointer[--len] = 0;
-
-    while(* pointer && isspace(* pointer)) {
-        ++pointer;
-        --len;
-    }
-
-    memmove(str, pointer, len + 1);
-}
-
-
 static char* get_system_architecture() {
     struct utsname buffer;
     char * arch = NULL;
@@ -649,20 +630,6 @@ static bool find_string_in_file(const char *path, const char *pattern) {
 }
 
 
-static bool is_file_empty(const char *file) {
-    struct stat stbuf;
-
-    if (stat(file, &stbuf) == -1) {
-        fprintf(log_handle, "can't access %s\n", file);
-        return false;
-    }
-    if ((stbuf.st_mode & S_IFMT) && ! stbuf.st_size)
-        return true;
-
-    return false;
-}
-
-
 static bool has_cmdline_option(const char *option)
 {
     return (find_string_in_file("/proc/cmdline", option));
@@ -696,43 +663,6 @@ static prime_intel_drv get_prime_intel_driver() {
 }
 
 
-static bool copy_file(const char *src_path, const char *dst_path)
-{
-    _cleanup_fclose_ FILE *src = NULL;
-    _cleanup_fclose_ FILE *dst = NULL;
-    int src_fd, dst_fd;
-    int n = 0;
-    char buf[BUFSIZ];
-
-    src = fopen(src_path, "r");
-    if (src == NULL) {
-        fprintf(log_handle, "error: can't open %s for reading\n", src_path);
-        return false;
-    }
-
-    dst = fopen(dst_path, "w");
-    if (dst == NULL) {
-        fprintf(log_handle, "error: can't open %s for writing.\n",
-                dst_path);
-        return false;
-    }
-
-    src_fd = fileno(src);
-    dst_fd = fileno(dst);
-
-    fprintf(log_handle, "copying %s to %s...\n", src_path, dst_path);
-
-    while ((n = read(src_fd, buf, BUFSIZ)) > 0)
-        if (write(dst_fd, buf, n) != n) {
-            fprintf(log_handle, "write error on file %s\n", dst_path);
-            return false;
-        }
-
-    fprintf(log_handle, "%s was copied successfully to %s\n", src_path, dst_path);
-    return true;
-}
-
-
 /* Get prime action, which can be "on", "off", or "on-demand" */
 static void get_prime_action() {
     char line[100];
@@ -1175,37 +1105,6 @@ static bool is_file(char *file) {
 }
 
 
-static bool is_dir(char *directory) {
-    struct stat stbuf;
-
-    if (stat(directory, &stbuf) == -1) {
-        fprintf(log_handle, "Error: can't access %s\n", directory);
-        return false;
-    }
-    if ((stbuf.st_mode & S_IFMT) == S_IFDIR)
-        return true;
-    return false;
-}
-
-
-static bool is_dir_empty(char *directory) {
-    int n = 0;
-    struct dirent *d;
-    DIR *dir = opendir(directory);
-    if (dir == NULL)
-        return true;
-    while ((d = readdir(dir)) != NULL) {
-        if(++n > 2)
-        break;
-    }
-    closedir(dir);
-    if (n <= 2)
-        return true;
-    else
-        return false;
-}
-
-
 static bool is_link(char *file) {
     struct stat stbuf;
 
@@ -2529,6 +2428,8 @@ int main(int argc, char *argv[]) {
     /* Device information (discrete) */
     struct device discrete_device = { 0 };
 
+    /* Get nvidia driver version */
+    int major, minor, extra;
 
     while (1) {
         static struct option long_options[] =
@@ -3003,6 +2904,13 @@ int main(int argc, char *argv[]) {
         for (i = 0; i < 500; i++, usleep(20000)) {
             if (is_file(UDEV_NVIDIA_COMPLETED))
                 break;
+            /* Get nvidia driver version inside loop because sometime nvidia
+             * driver take longer time to load.
+             */
+            if (get_nvidia_driver_version(&major, &minor, &extra) &&
+                major < 390) {
+                break;
+            }
         }
         fprintf(log_handle, "Takes %dms to wait for nvidia udev rules completed.\n", i * 20);
     }
